if acrobatics --c'mon
	print("Acrobatics is already loaded - don't load both of these files at once!")
	return
end

rawset(_G, "CONFIG_Queue", CONFIG_Queue or {})

local function CONFIG_RegisterVar(var)
	table.insert(CONFIG_Queue, var)
	return CV_RegisterVar(var)
end

-- GAUGE
local TRICKTIMERMAX = 70

local GAUGE_WIDTH = 50
local GAUGE_HEIGHT = 4

local GAUGE_SPACE = 1
-- GAUGE

local map_acrofix = {}
local acrofix_value

--freeslots
freeslot("sfx_tric", "sfx_trice")

-- 1 - vanilla
-- 2 - custom
local sounds = {{}, {}}

for tsnd = 1, 5 do --trick confirms
	sounds[1][tsnd] = freeslot("sfx_acgr"..tsnd)
	sounds[2][tsnd] = freeslot("sfx_aacgr"..tsnd)
end

local cv_tricksound = CONFIG_RegisterVar({
	name = "as_tricksound",
	defaultvalue = "Vanilla",
	possiblevalue = { Vanilla = 1, SonicColors = 2, },

	config_menu = "Acrobasics",
	displayname = "Trick Sound",
	description = "Choose of sound that plays when getting boost from trick.",
})

local function playTrickSound(player, num)
	-- Vanilla acro doesn't play those two for some reason.
	-- Also <3 ^w^
	if num <3 and cv_tricksound.value == 1 then return end

	local sfx = sounds[cv_tricksound.value][num]

	if sfx then
		S_StartSound(nil, sfx, player)
	end
end

--CVARS--

local cv_trickgauge = CONFIG_RegisterVar({
    name = "as_trickgauge",
    defaultvalue = "On",
    possiblevalue = CV_OnOff,

	config_menu = "Acrobasics",
	displayname = "Trick Gauge",
	description = "Display gauge which shows timer before trick chain expires.",
})

local cv_trickdhudtransparency = CONFIG_RegisterVar({
	name = "as_trickhudtransparency",
	defaultvalue = "Off",
	possiblevalue = {
		Off = 0,
		Classic = 1,
		Hud = 2,
	},

	config_menu = "Acrobasics",
	displayname = "Trick Hud Transparency",
	description = "Transparency for trick hud effects.",
})

local cv_trickhudcolor = CONFIG_RegisterVar({
    name = "as_trickhudcolor",
    defaultvalue = "Off",
    possiblevalue = CV_OnOff,

    config_menu = "Acrobasics",
    displayname = "Trick HUD Color",
	description = "Colorize trick HUD background. Inverted if colorizedhud is enabled."
})

local as_minairframes = CV_RegisterVar({
	name = "as_minairframes",
	defaultvalue = "0",
	flags = CV_NETVAR,
	possiblevalue = CV_Unsigned,
})

local as_active = CV_RegisterVar({
	name = "as_active",
	defaultvalue = "On",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
})

local as_trickreminder = CONFIG_RegisterVar({
	name = "as_trickreminder",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,

	config_menu = "Acrobasics",
	displayname = "Trick Reminder",
	description = "Display a reminder in chat every time race starts.",
})

local as_trickyield = CV_RegisterVar({
	name = "as_trickyield",
	defaultvalue = "combo",
	flags = CV_NETVAR,
	possiblevalue = {combo = 0, sneaker = 1}
})

local as_wildspeed = CV_RegisterVar({
	name = "as_wildspeed",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff,
})

local as_fixanimations = CONFIG_RegisterVar({
	name = "as_fixanimations",
	defaultvalue = "On",
	possiblevalue = CV_OnOff,

	config_menu = "Acrobasics",
	displayname = "Fix animations",
	description = "Fixes the broken HUD animations."
})

-- Added manually in sungarden's hostmod
--addHook("ThinkFrame", function()
--	if not as_hmload and (CV_FindVar("hm_scoreboard")) then --check for hostmod
--		HM_Scoreboard_AddMod({disp = "Acrobatics(L)", var = "as_active"})
--		as_hmload = true
--	end
--end)

-- Not sure if this thing is portable across platforms (lua reference manual
-- says that char codes aren't. But every encoding have numbers in same order,
-- right?..)
local function isdigit(byte)
    return byte >= string.byte("0") and byte <= string.byte("9")
end

local function getMapId(name)
    local name = name:lower()

    if name:find("map") then
        name = name:sub(4, 5)
    end

    local x = name:byte(1)
    local y = name:byte(2)

    if x == nil or y == nil then return 0 end

    if isdigit(x) then
        if isdigit(y) then
            return (x - string.byte("0")) * 10 + y - string.byte(0)
        end

        return 0
    end

    local q = x - string.byte("a")
    local p

    if isdigit(y) then
        p = y - string.byte("0")
    else
        p = 10 + y - string.byte("a")
    end

    return (36*q + p) + 100
end

local buttonnames = {
	[BT_BRAKE] = "BRAKE",
	[BT_CUSTOM1] = "CUSTOM1"
}

COM_AddCommand("as_trickbtn", function(p, arg)
	arg = tostring(arg):lower()
	local newbutton
	if arg == "brake" then
		newbutton = BT_BRAKE
	elseif arg == "custom" or arg == "custom1" then
		newbutton = BT_CUSTOM1
	else
		CONS_Printf(p, "\135* Please choose between \130BRAKE \135or \130CUSTOM.")
		return
	end

	if tsrb2kr --TEAM SONIC RACING: Uses all the Customs, so you shouldn't change this, probably.
		CONS_Printf(p, "\133*WARNING:\131 TSRB2KR is active, CUSTOM1 cannot be used!")
		CONS_Printf(p, "\131* Trick button force changed to \130BRAKE\131.")
		newbutton = BT_BRAKE
	else
		CONS_Printf(p, "\131* Trick button changed to \130" .. buttonnames[newbutton] .. "\131.")
	end

	p.trick_button = newbutton
end)

local function setmapacrofix(p, map_name, value)
    if not map_name then
        CONS_Printf(p, "Usage: as_setmapacrofix <map> [value]")
        CONS_Printf(p, "To make map use the default value, use without second argument")
        return
    end

    local map_id = getMapId(map_name)

    if value then
        map_acrofix[map_id] = tonumber(value)
        CONS_Printf(p, "Set acrofix value for map "..map_name.." to "..value)
    else
        map_acrofix[map_id] = nil
        CONS_Printf(p, "Map "..map_name.." will use default acrofix value")
    end

    if gamemap == map_id then
        if map_acrofix[map_id] ~= nil then
            acrofix_value = map_acrofix[map_id]
        else
            acrofix_value = as_minairframes.value
        end
    end
end

COM_AddCommand("as_setmapacrofix", setmapacrofix, COM_ADMIN)

--MATH and OTHER STUFF--

--rescale from ashnal- saved a braincell
local function rescale(value, oldmin, oldmax, newmin, newmax)
    return newmin +  FixedMul(FixedDiv( value-oldmin , oldmax-oldmin), newmax-newmin)
end

--TIME FOR TRICKS--
local function setplayertrickvars(p)
	p.acrostuff = {
		--hastricked = false, --self explanatory
		spintimer = 0, -- How long the 'spin' animation will play when you've done a trick.
		spinning = (p.mo.state == S_KART_SPIN or p.mo.state == S_KART_PAIN), --If you're already spinning, there's no reason to force things.
		lastz = p.mo.z, -- Race tricks only work ascending.
		airframes = 0, -- For how long player been in air for trick
		trickchaintimer = 0, -- Counts down every tic, for combo tricks
		inchain = false, -- Eligibility for chaining combo
		chaincount = 0, -- Count your combos!
		--trickboost = 0, -- Time-based [Low speed, max length] - integers
		--trickbooststr = 0, --Strength based [Low weight, max strength] - fixed point
		trickboostdust = 0, --Timer for dust
		trickoffroad = 0, --Needs its own timer so great tricks don't over frontrun
		tricksneakerlastframe = 0, --Post storage checks
		tricklastframebtn = 0, --Post storage checks
		tricksneaker = false, --flag for sneaker vs trick boosts
		tricksneakerbuffer = 0, --a more specific sneaker check
		tricksneakerprevboost = 0, --fixed point, for value smoothing between sneakers and tricks
		--finaltrickboost = 0, --fixed point for value smoothing
		--sincelasttrick = 0, --Timer for style messages.
		--trickgrade = 0, --Usually syncs up to chargecheck, for style messages

		tricklastitem = 0,
		trickprevitemcount = 0,
		stomptrick = false,
	}

	-- which variables do we have to keep?
	-- chronoshift
	p.hastricked = false
	-- booststack
	p.trickboost = 0
	p.trickbooststr = 0
	p.finaltrickboost = 0
	-- the lamb taunts
	p.sincelasttrick = 0
	p.trickgrade = 0
	-- just needs to survive map changes
	p.trick_button = $ or BT_BRAKE
end

local function resettrickboost(p)
	local as = p.acrostuff
    as.chaincount = 0
    p.trickgrade = 0
    p.trickboost = 0
    p.trickbooststr = 0
    p.finaltrickboost = 0
end

local function cancelTrickMidair(player)
	if player.mo and not P_IsObjectOnGround(player.mo) then
		local tricked = player.hastricked
		setplayertrickvars(player)
		player.sincelasttrick = tricked and TICRATE or 0
	end
end

addHook("PlayerSpin", cancelTrickMidair)
addHook("PlayerExplode", cancelTrickMidair)
addHook("PlayerSquish", cancelTrickMidair)

local function netvars(net)
    map_acrofix = net(map_acrofix)
    acrofix_value = net(acrofix_value)
end

addHook("NetVars", netvars)

local function mapload(mapid)
    if map_acrofix[mapid] ~= nil then
        acrofix_value = map_acrofix[mapid]
    else
        acrofix_value = as_minairframes.value
    end
end

addHook("MapLoad", mapload)

--TRICK FUNCTIONS

local function tricksparkle(p, potential) --The sparkle function. potential [boolean] denotes intensity: possible chain/charging.
	if(not p.kartstuff[k_invincibilitytimer]) and not(p.powers[pw_nocontrol])
		local pmo = p.mo
		if potential == true
			K_SpawnSparkleTrail(pmo) --maybe i'll write a real one later that's not this
		else
			local sparkle = P_SpawnMobj(pmo.x+P_RandomRange(-30, 30)*pmo.scale, pmo.y+P_RandomRange(-30, 30)*pmo.scale, pmo.z+P_RandomRange(-15, 0)*pmo.scale, MT_SPARKLETRAIL)
			sparkle.destscale = pmo.destscale
			sparkle.target = pmo
			sparkle.color = pmo.color
		end
		if leveltime%4 > 2
			pmo.colorized = true
		else
			pmo.colorized = false
		end
	end
end

---TRICKBOOST FUNCTIONS
--DUST AND EFFECTS

local function K_TrickDustRelease(p, intensity) --since i can't get into spawn dust funcss on lua
	local pmo = p.mo
	local ks = p.kartstuff

	--3 Checks -- Ground Check, Way Too Slow, or you're already kicking up sliptide dust.
	if not P_IsObjectOnGround(pmo) or FixedDiv(p.speed, mapobjectscale) <= 25*FRACUNIT or ks[k_aizdriftstrat] then
		return --yeah
	end

	if leveltime%2 == 1 or intensity == 3--Kick up some dust every other frame, like the game
		for i = 0,1 do
			local trvangle = pmo.angle - (ANGLE_45/5)*ks[k_drift]
			local newx = pmo.x + P_ReturnThrustX(pmo, trvangle + ((i == 0) and -1 or 1)*ANGLE_90, FixedMul(24*FRACUNIT, pmo.scale))
			local newy = pmo.y + P_ReturnThrustY(pmo, trvangle + ((i == 0) and -1 or 1)*ANGLE_90, FixedMul(24*FRACUNIT, pmo.scale))
			local dust = P_SpawnMobj(newx, newy, (verticalflip and (pmo.ceilingz or -1) or pmo.floorz), MT_AIZDRIFTSTRAT)
			dust.target = pmo
			dust.angle = trvangle - ((i == 0) and -1 or 1)*ANGLE_67h
			dust.scale = pmo.scale/(5-intensity)
			dust.destscale = 2*pmo.scale/(6-intensity)
			dust.scalespeed = pmo.scale/6
			dust.momx = 3*pmo.momx/5
			dust.momy = 3*pmo.momy/5
			//dust.fuse = 25 -- dunno if this actually does shit
		end
	end
end


--TRICKCALCS
local function calctrickboostlength(p, debug) --use debug to force values
	local as = p.acrostuff
	local um = rescale(p.kartspeed<<FRACBITS, 9<<FRACBITS, 1<<FRACBITS, 5<<FRACBITS, 15<<FRACBITS)
	um = $>>FRACBITS
	return (um*as.chaincount)+10 --Capping a trickboost
end


--HUD Animation Functions

--[[
local flavorframeinfo = {
		[-1] = 8,
		[0] = 8,
		[1] = 10,
		[2] = 16,
		[3] = 8,
		[4] = 8,
		[5] = 8,
		}
local curframe, maxframes
local function flavoranimation(p) --TODO: Sometimes the animation bugs the fuck out on longer anims
	curframe = 0
	maxframes = flavorframeinfo[p.trickgrade] --The max amount of frames for each status
	curframe = (leveltime)%maxframes
	return curframe	+ 1 --[tl note: i am too lazy to rename my exports from aseprite]
end
--]]

local function useColorHud(v)
    if v.useColorHud then return v.useColorHud() ~= (cv_trickhudcolor.value == 1) end
    return cv_trickhudcolor.value
end

local function getHudColor(v, p)
    if v.getHudColor then return v.getHudColor() end

    return p.skincolor
end

local function getHudColormap(v, p)
    if not useColorHud(v) then return end

    return v.getColormap(TC_DEFAULT, getHudColor(v, p))
end

local function getTrickPatch(v, grade)
    if useColorHud(v) then return v.cachePatch("TSRIBOC"..grade) end

    return v.cachePatch("TSRIBON"..grade)
end

local text = { [0] = "MISS...", "OK...", "NICE!", "RADICAL!", "WILD!" }
local strwidths = { [0] = 76, 50, 56, 92, 62 }
local font
local width, offset = {}, {}
local function newflavoranimation(v, p, x, y, flags)
	if not font then
		font = { [33] = v.cachePatch("TSFNTEXC"), [46] = v.cachePatch("TSFNTDOT") }
		for i = 65, 90 do -- A-Z
			font[i] = v.cachePatch(string.format("MKFNT%03d", i))
		end
		for k, v in pairs(font) do
			width[k] = (v.width + 1)*FRACUNIT
			offset[k] = 0
		end
		-- I...
		width[73] = (font[73].width - 1)*FRACUNIT
		offset[73] = -FRACUNIT
	end

	if v.interpolate then v.interpolate(p.trickgrade) end

	local str = text[p.trickgrade]
	-- can't do a stringWidth here because consistency am i right
	local w = strwidths[p.trickgrade]*FRACUNIT

	v.draw(x, y, getTrickPatch(v, p.trickgrade), flags, getHudColormap(v, p))
	x = (x+1)*FRACUNIT - w/2
	y = (y-7)*FRACUNIT

	for i = 1, #str do
		local xofs, yofs = 0, 0
		if p.trickgrade == 0 then -- MISS...
			-- it's literally random what the fuck am i supposed to do
			local xoffsets = {
				{ 2,  1,  0,  1,  2,  1,  0,  1, }, -- M
				{ 2,  3,  4,  2,  0,  0,  0,  1, }, -- I
				{ 2,  2,  2,  0,  -2, 0,  2,  2, }, -- S
				{ 2,  1,  0,  0,  0,  1,  2,  2, }, -- S
				{ 0,  1,  2,  0,  -2, -2, -2, -1,}, -- .
				{ -2, -2, -2, -4, -6, -5, -4, -3,}, -- .
				{ -4, -6, -8, -8, -8, -6, -4, -4,}, -- .
			}
			local yoffsets = {
				{ 0,  1,  2,  3,  4,  2,  0,  0, }, -- M
				{ -4, -5, -6, -6, -6, -4, -2, -3,}, -- I
				{ 0,  2,  4,  3,  2,  2,  2,  1, }, -- S
				{ 4,  2,  0,  2,  4,  2,  0,  2, }, -- S
				{ 0,  1,  2,  2,  2,  0,  -2, -1,}, -- .
				{ -4, -5, -6, -6, -6, -4, -2, -3,}, -- .
				{ 4,  4,  4,  2,  0,  -1, -2, 1, }, -- .
			}
			local t = (leveltime % 8) + 1
			xofs = xoffsets[i][t]*FRACUNIT/2
			yofs = yoffsets[i][t]*FRACUNIT/2
		elseif p.trickgrade == 1 then -- OK...
			local lt = leveltime
			if not as_fixanimations.value and lt % 10 == 9 then
				lt = lt + 1
			end
			local t = (lt + (5-i)*2) % 10
			yofs = -abs(t - (t >= 5 and 10 or 0)) * FRACUNIT/2
		elseif p.trickgrade == 2 then -- NICE!
			local offsets = { [0] = 0,  2,  4,  5,  6,  5,  4,  2,
			                        0,  -2, -4, -5, -6, -5, -4, -2, }
			yofs = offsets[leveltime % 16]*FRACUNIT/2
			if not as_fixanimations.value and leveltime%16 >= 8 then
				yofs = 0
			elseif i%2 then
				yofs = -yofs
			end
		elseif p.trickgrade == 3 then -- RADICAL!
			local yoffsets = { [0] = 0, 1, 2, 1, 0, -1, -2, -1 }
			local xoffsets = { [0] = 0, -1, -2, -3, -4, -3, -2, -1 }
			yofs = yoffsets[leveltime % 8]*FRACUNIT/2
			xofs = xoffsets[leveltime % 8]*FRACUNIT/2
			if i%2 then
				yofs = -yofs
			end
		elseif p.trickgrade == 4 then -- WILD!
			local offsets = { [0] = 0, -1, -2, -3, -4, -3, -2, -1 }
			yofs = offsets[leveltime % 8]*FRACUNIT/2
			xofs = yofs*(-i + 3) -- the W is one pixel off at the farthest but who tf cares
		end
		local char = str:byte(i)
		v.drawScaled(x + offset[char] + xofs, y + yofs, FRACUNIT, font[char], flags)
		x = x + width[char]
	end

	if v.interpolate then v.interpolate(false) end
end

local function AS_ApplyTrickBoost(p, nocheckground, deltatracks)
	local pmo = p.mo
    local as = p.acrostuff
    if not as_trickyield.value then
        if as_wildspeed.value then
            as.trickchaintimer = 52
            as.inchain = true
            p.trickgrade = 4
            as.chaincount = max(as.chaincount or 0, 5)
        end

        p.trickboost = calctrickboostlength(p)
        as.trickoffroad = min(p.trickboost, 6+(2*as.chaincount))
        as.trickboostdust = max(p.trickboost - 10, 10)
        p.kartstuff[k_speedboost] = max($, ((3+min(as.chaincount,5))*FRACUNIT)/10)

        if p.hastricked and (nocheckground or P_IsObjectOnGround(pmo)) then
            if as.trickchaintimer and as.chaincount > 1 and as.inchain then --Midcombo?
                if not nocheckground or deltatracks then playTrickSound(p, min(as.chaincount,4)+1) end
                S_StartSound(pmo, sfx_s23c)
                K_TrickDustRelease(p, 3)

                -- Why do it like that? It used to add 18 when you press trick button and cap it at 52,
                -- and then when you land it adds another 18 and caps at 18. Fine with usual display but looks
                -- weird with gauge. So, if done this way gauge fills up same way as before but is also looks fine

                -- Changed to add 16 at first because of weird bug when you get 2 radicals in row on one map because
                -- of timer changes, which made me add 2 tics in air
                as.trickchaintimer = min(min($ + 16, 52) + 18, 70)

                p.trickgrade = min($ + 1, 4)

                if as.chaincount >= 4 then
                    K_PlayPowerGloatSound(pmo)
                else
                    if P_RandomChance((as.chaincount*FRACUNIT)/7) then K_PlayBoostTaunt(pmo) end
                end
            else --Let's start a combo, then
				playTrickSound(p, 1)
                S_StartSound(pmo, sfx_s23c)
                as.trickchaintimer = 70
            end
        end

    else
        K_DoSneaker(p, 1) -- kind of funny how small this is
    end
    p.sincelasttrick = as.trickchaintimer

    if p.hastricked and (deltatracks or P_IsObjectOnGround(pmo)) then
        p.hastricked = false
    end

    as.stomptrick = false

    p.frameangle = pmo.angle
    pmo.color = p.skincolor
end

addHook("MapLoad", do
    for p in players.iterate do
		p.acrostuff = nil
		if p.mo and not p.spectator then
			setplayertrickvars(p) --Reset it all
		end
    end
end)

local starttime = 6*TICRATE + (3*TICRATE/4) --Everyone's doing it

local function stompTrick(pmo, mo)
    -- I fucking hate do that every time
    if (pmo.z + pmo.height < mo.z) or (mo.z + mo.height < pmo.z) then return end

    if not pmo.player.hastricked then return end
    if not (mo.flags & MF_SOLID) then return end

    local mapobjectscale = mapobjectscale

    local player = pmo.player
    local otherplayer = mo.player

    if otherplayer and (player.kartstuff[k_squishedtimer] or player.kartstuff[k_hyudorotimer]
				or player.kartstuff[k_justbumped] or pmo.scale > mo.scale + (mapobjectscale/8)
				or player.powers[pw_flashing]
				or otherplayer.kartstuff[k_squishedtimer] or otherplayer.kartstuff[k_hyudorotimer]
				or otherplayer.powers[pw_flashing]
				or otherplayer.kartstuff[k_justbumped] or mo.scale > pmo.scale + (mapobjectscale/8)) then return end

    --print(string.format("P_IsObjectOnGround(mo) = %s", P_IsObjectOnGround(mo) and "true" or "false"))
    --print(string.format("P_IsObjectOnGround(pmo) = %s", P_IsObjectOnGround(pmo) and "true" or "false"))
    --print(string.format("pmo.momz = %d", pmo.momz))
    --print(string.format("player.kartstuff[k_justbumped] = %d", player.kartstuff[k_justbumped]))

    if P_IsObjectOnGround(mo) and not P_IsObjectOnGround(pmo) and pmo.momz < 0 and not player.kartstuff[k_justbumped] then
        AS_ApplyTrickBoost(player, true)
        local as = player.acrostuff
        as.stomptrick = true
        player.hastricked = true
        as.inchain = true
    end
end

addHook("MobjMoveCollide", stompTrick, MT_PLAYER)

addHook("MobjCollide", function(mo, pmo)
	if pmo.player then return stompTrick(pmo, mo) end
end, MT_PLAYER)

--The ThinkFrame

local deltapatched = false
addHook("ThinkFrame", function()
	-- replace the outdated trickboost in deltatrack's waterskip momentumfix lua
	if not deltapatched then
		if funny_waterskip_LMAO then
			funny_waterskip_LMAO.SetAcrobasicsTrick = function(p)
				if p.hastricked then return AS_ApplyTrickBoost(p, true, true) end
			end
		end
		deltapatched = true
	end

	if G_BattleGametype() then return end -- nah.

    for p in players.iterate do
		local pmo = p.mo
		if not pmo then continue end

		if leveltime == TICRATE*7 and as_wildspeed.value then
			S_StartSound(pmo, sfx_kgloat)
		end

        // Initialization etc etc durr hurr
		if leveltime == 1 or not p.acrostuff then setplayertrickvars(p) end

		local as = p.acrostuff
		local ks = p.kartstuff

		if P_IsObjectOnGround(pmo) then
			as.airframes = 0
		else
			as.airframes = $ + 1
		end

		if leveltime == 2*TICRATE and as_trickreminder.value and as_active.value then
			local pref_btn_str = buttonnames[p.trick_button]
			if not G_BattleGametype()
				chatprintf(p, "\131*Press \130" .. pref_btn_str .. "\131 in midair to perform a \130trick! \130 Chain multiple tricks for \131speed!", true)
			else
				chatprintf(p, "\131*Tricks are \133disabled\131 in battle mode.", true)
			end
		end

		if leveltime <= starttime then continue end --moved here because otherwise we can't initialize hurrr

		-- Kill trick states if you die, dingus
		if (p.health < 1 or p.playerstate == PST_DEAD)
			if p.chargeactive or as.trickchaintimer then
				p.trickgrade = 0 -- Let's rub salt in that wound
				p.sincelasttrick = 8
			end
			p.hastricked = false
			as.trickchaintimer = 0
			as.inchain = false
			p.trickboost = 0
			p.trickbooststr = 0
			continue --the difference between continue and break could save someone's life
		end

		-- Ground checks: Tick down the timer for combo tricks, reset spin timers.
		if P_IsObjectOnGround(pmo) then
			as.spintimer = 0
			if as.trickchaintimer then
				as.trickchaintimer = $ - 1
				as.inchain = true
			elseif as.inchain then
				as.inchain = false
			end
		end

-- Pre Trick Block in Airtime

		-- can they trick?
		if as_active.value and
		not (P_IsObjectOnGround(pmo)
		or ks[k_respawn]
		or ks[k_spinouttimer]
		or p.playerstate == PST_DEAD
		or pmo.drifthold
		or p.powers[pw_nocontrol]
		or as.airframes < acrofix_value) then
			if not p.hastricked or as.stomptrick then
				if p.cmd.buttons & p.trick_button and ((pmo.z > as.lastz + 1) or (pmo.z < as.lastz - 1 and pmo.eflags & MFE_VERTICALFLIP)) then-- shoutouts to simpleflipgoop
					p.hastricked = true
                    as.stomptrick = false
					if as.trickchaintimer then --We're mid-combo
						as.inchain = true
						S_StopSoundByID(pmo, sfx_tric)
						S_StopSoundByID(pmo, sfx_trice) --:oh:
						S_StartSound(pmo, sfx_trice)
						as.chaincount = $ + 1
                        as.trickchaintimer = min($+2, 70) -- Preventing weird "double nice" trick
					else --We need to start the combo.
                        resettrickboost(p)
						S_StartSound(pmo, sfx_tric)
						as.inchain = true
						p.trickgrade = 1
						as.chaincount = 1
					end
					if not (ks[k_pogospring] or as.spinning) then
						as.spintimer = 30 --vinny.mp3
					end
				end
			else -- Post-press block
				--Revert if you happen to get shot down midair.
				if ks[k_spinouttimer] then
					as.trickchaintimer = 0
					as.inchain = false
				end

				if as.spintimer then
					local spinfactor = as.chaincount
					local spindiff = ( ANGLE_11hh * leveltime * (not spinfactor and 6 or spinfactor) ) --TODO
					as.spintimer = max($ - 1, 0)
					p.frameangle = $ + spindiff
					tricksparkle(p, true)
				elseif not (ks[k_pogospring] or as.spinning) then
                    p.frameangle = pmo.angle
                end
            end
        end


--Visuals and Sparkles --Combo
		if as.trickchaintimer or p.hastricked and not (as.spintimer or p.trickboost) then
			tricksparkle(p, false)
		end

--Trick Landing Block.
		if (P_IsObjectOnGround(pmo) and (p.hastricked or as.stomptrick))
		or (as_wildspeed.value and not (ks[k_spinouttimer] or ks[k_wipeoutslow])) then
			AS_ApplyTrickBoost(p)
		end

		-- tricksneakerCheck

		--A trick JUST landed and we suddenly have trickboost, and we're not midsneaker buffer.
		if p.trickboost > 0 and not(as.tricksneakerbuffer)
			as.tricksneaker = true
		end
		--The following blocks try to prove otherwise.

		if (ks[k_rocketsneakertimer] and as.tricklastframebtn == 0 and (p.cmd.buttons & BT_ATTACK)) and as.tricklastitem ~= KITEM_ROCKETSNEAKER
			as.tricksneakerbuffer = ks[k_sneakertimer] --A buffer that means "we're in a sneaker now don't disturb" -number from sourcecode
			as.tricksneaker = false	--It was PRESSED, - we want the real sneakerboost.
		end

		if ks[k_floorboost] --Sneaker panel?
			as.tricksneakerbuffer = ks[k_sneakertimer]
			as.tricksneaker = false
		end

		--Check for a sneakertimer change of ANYTHING.
		if ks[k_sneakertimer] ~= as.tricksneakerlastframe then
			--First - item use
			if (as.tricklastitem == KITEM_SNEAKER and ks[k_itemtype] ~= KITEM_SNEAKER) or --Sneaker
			(ks[k_itemtype] == KITEM_SNEAKER and ks[k_itemamount] < as.trickprevitemcount) then --Multisneaker
				as.tricksneakerbuffer = ks[k_sneakertimer]
				as.tricksneaker = false
			end

			if (ks[k_sneakertimer] > as.tricksneakerlastframe) --Something else is making us sneaker and we need to not worry about it
				as.tricksneakerbuffer = ks[k_sneakertimer]
				as.tricksneaker = false
			end
		end
		-- end tricksneakerCheck

		-- applytrickboost
		if p.trickboost > 0 then
			if as.tricksneaker then ks[k_sneakertimer] = max($, p.trickboost) end --Sometimes trickboosts are longer than sneakers, so let's just give 'em the whole thing.
			--Calculating trick strength via kartspeed.
			local trickbooststr = rescale(p.kartspeed*FRACUNIT, 9*FRACUNIT, FRACUNIT, 3*FRACUNIT/20, 3*FRACUNIT/8) --Min 15%[9], Max 37.5%[1]
			trickbooststr = $*(min(as.chaincount, 4)) --It can get out of control!

			--Offroad penalty after leniency is over -- bonus TANKS if you're still offroad to prevent accels from taking shortcuts
			if not as.trickoffroad and ks[k_offroad] > FRACUNIT and not as.tricksneakerbuffer then
				trickbooststr = 0 --nope
				ks[k_sneakertimer] = 0 --Reapply offroad pain if no true sneaker
			end

			local finaltrickboost = trickbooststr --let's set this aside for now.

			--Applying the boost is way less straightforward because a trickboost is less then a sneaker boost [which is being applied by sneakertimer] - so we have to undo sneakerboost beforehand carefully.
			if as.tricksneaker and not(as.tricksneakerbuffer) then  --we're in puretrick boost!

				if not(tsrb2kr and pmo.tsr_ultimateon)
					ks[k_accelboost] = $ - 8*FRACUNIT -- Nuke accel.
				end

				if as.tricksneakerprevboost > trickbooststr then-- ...but readd a little of it, if we're transitioning. value smoothing from the other way.
					finaltrickboost = as.tricksneakerprevboost + (trickbooststr - as.tricksneakerprevboost)/(TICRATE/2);
				end

			end

			p.trickbooststr = trickbooststr
			p.finaltrickboost = finaltrickboost

			if not(as.tricksneakerbuffer)
				if tsrb2kr and pmo.tsr_ultimateon
					ks[k_speedboost] = $ + (trickbooststr/10) --What, is wipezones level speed not enough for you?
				else
					ks[k_speedboost] = finaltrickboost --forced here deliberately.
				end
			end
			ks[k_accelboost] = max($, 4*FRACUNIT) --400% accel increase for nonsneaker boosts.
			if as.trickboostdust >= 24
				K_TrickDustRelease(p, 1)
			elseif as.trickboostdust >= 10
				K_TrickDustRelease(p, 0)
			else
				--lmao
			end
			tricksparkle(p, false) --kirakira
			as.trickboostdust = max($ - 1, 0)
			p.trickboost = max($ - 1,0)
			as.trickoffroad = max($ - 1,0)
		end

		as.tricksneakerprevboost = p.finaltrickboost --stored for next tic because sneakertimer's gonna overwrite it :deadinside:
		as.tricksneakerbuffer = max($ - 1, 0) --This needs to be run outside the trickboost check so we can have overlaps of trick vs sneaker.
		-- end applytrickboost

        as.lastz = pmo.z --For trick
		as.tricklastitem = ks[k_itemtype]
		as.trickprevitemcount = ks[k_itemamount]
		as.tricksneakerlastframe = ks[k_sneakertimer]
		as.tricklastframebtn = (p.cmd.buttons & BT_ATTACK) and ($+1) or 0

		if not as.trickchaintimer then
			p.sincelasttrick = 0
		else
			p.sincelasttrick = max($ - 1, 0)
		end
    end
end)

local function drawGauge(v, x, y, width, height, fill, flags)
    local fill_width = FixedMul((width - GAUGE_SPACE*2)*FRACUNIT, fill)/FRACUNIT
    local fill_height = GAUGE_HEIGHT - GAUGE_SPACE*2

	local gaugecolor = 160

	if fill < FRACUNIT/4 then
		gaugecolor = 130
	elseif fill < FRACUNIT/2 then
		gaugecolor = 100
    end

    local gaugebgcolor = 24

    if useColorHud(v) and SG_Translations then
        gaugebgcolor = SG_Translations[getHudColor(v, p)][8]
    end

	v.drawFill(x, y, width, height, gaugebgcolor|flags)
	if v.interpolate then v.interpolate(true) end
	v.drawFill(x + GAUGE_SPACE, y + GAUGE_SPACE, fill_width, fill_height, gaugecolor|flags)
	if v.interpolate then v.interpolate(false) end
end

local meterx, metery, trickflags, flavoralignment, xspscr, yspscr

local splitflavorinfo = {
	[0] = "MISS...",
	[1] = "OK...",
	[2] = "NICE!",
	[3] = "RADICAL!",
	[4] = "WILD!",
	}
local splitcolorinfo = {
	[0] = V_GRAYMAP,
	[1] = V_ORANGEMAP,
	[2] = V_YELLOWMAP,
	[3] = V_GREENMAP,
	[4] = V_SKYMAP,
}

hud.add(function(v, p, c)
	if not as_active.value then return end
    if replayfreecam then return end
	local as = p.acrostuff
	if not as or p.exiting then return end --let's not get hasty here
	meterx = 190 --for real screens
	metery = 48
	trickflags = V_10TRANS
	flavoralignment = 100*0

    local tricktimer = min(as.trickchaintimer, TRICKTIMERMAX)
    local fill = FixedDiv(tricktimer*FRACUNIT, TRICKTIMERMAX*FRACUNIT)

	--Welcome to Flavorblock
    if (cv_trickdhudtransparency.value == 1) and as.trickchaintimer then
        --I think a fadeout would be better - Latius
		trickflags = V_10TRANS
    	local fademod = 11 - min((p.sincelasttrick * 16) / max(as.trickchaintimer, 1), 9)
    	trickflags = $ * fademod
	elseif cv_trickdhudtransparency.value == 2 then
		trickflags = V_HUDTRANS
	elseif cv_trickdhudtransparency.value == 0 then
		trickflags = trickflags & ~V_ALPHAMASK
	end

	if not(splitscreen) then
		trickflags = $ | V_SNAPTOTOP
		if p.sincelasttrick > 0 or (cv_trickgauge.value and tricktimer) then
			if cv_trickgauge.value and fill > 0 then
				drawGauge(v, 130, 47, GAUGE_WIDTH, GAUGE_HEIGHT, fill, trickflags)
			end

			--[[
			local patch = string.format("TS%dFLV%d", p.trickgrade, flavoranimation(p))
			if not as_fixanimations.value then
				patch = $:sub(1, 7)
			end
			v.draw(160+flavoralignment, 40, v.cachePatch(patch), trickflags) --my god
			--]]
			newflavoranimation(v, p, 160, 40, trickflags)
		end
	else
		--ss indexer by callmore
		local ssindex = c.pnum - 1
		--[[ ...another victim of undocumented lua variables
			for i = 0, splitscreen do
				if p == displayplayers[i] then
					ssindex = i
					break
				end
			end
		if ssindex == nil then return end --you're not visible and you need to go
		--]]

		local sox = 0 --splitoffsetx, not clothes
		local soy = 0 --splitoffsety, not tofu
		local flavorx = 4 --flavor text in splitscreen needs to be simple text... real estate is scarce!
		local flavory = 50
		trickflags = splitcolorinfo[p.trickgrade]
		if splitscreen == 1 --2p mode, big split
			flavorx = 6 --bothered me a little
			if ssindex == 1 --bottom
				soy = 100 --shifts to lower scr
			else --topscreen
				soy = 0
			end

		else --3/4p

			flavory = 30 --because the item boxes are smaller
			if ssindex%2 == 1
				sox = 160 --shift right!
			end
			if ssindex > 1
				soy = 100
			end

		end

		if p.sincelasttrick > 0 then
			if cv_trickgauge.value and fill > 0 then
				drawGauge(v, flavorx+sox, flavory+soy+10, GAUGE_WIDTH, GAUGE_HEIGHT, fill, trickflags)
			end

			v.drawString(flavorx+sox, flavory+soy, splitflavorinfo[p.trickgrade], trickflags) --a simpler time
		end

	end

end)


rawset(_G, "acrobasics", true) --Someone's gonna want it.
print("\131* Acrobasics v1, by Angular. Load successful! Please spin responsibly. *")
